@@ -21,14 +21,13 @@
  */
 package org.wildfly.clustering.server.dispatcher;
 
-import java.io.ByteArrayInputStream;
-import java.io.DataInputStream;
 import java.io.IOException;
 import java.net.InetSocketAddress;
+import java.nio.ByteBuffer;
 import java.time.Duration;
-import java.util.AbstractMap;
 import java.util.List;
 import java.util.Map;
+import java.util.NoSuchElementException;
 import java.util.Optional;
 import java.util.concurrent.ConcurrentHashMap;
 import java.util.concurrent.ConcurrentMap;
@@ -45,8 +44,9 @@ import org.jboss.as.clustering.context.DefaultExecutorService;
 import org.jboss.as.clustering.context.DefaultThreadFactory;
 import org.jboss.as.clustering.context.ExecutorServiceFactory;
 import org.jboss.as.clustering.logging.ClusteringLogger;
-import org.jboss.marshalling.Marshalling;
-import org.jboss.marshalling.Unmarshaller;
+import org.jboss.marshalling.MarshallingConfiguration;
+import org.jboss.marshalling.ModularClassResolver;
+import org.jboss.modules.ModuleLoader;
 import org.jgroups.Address;
 import org.jgroups.Event;
 import org.jgroups.JChannel;
@@ -68,8 +68,16 @@ import org.wildfly.clustering.group.Group;
 import org.wildfly.clustering.group.GroupListener;
 import org.wildfly.clustering.group.Membership;
 import org.wildfly.clustering.group.Node;
-import org.wildfly.clustering.marshalling.jboss.MarshallingContext;
-import org.wildfly.clustering.marshalling.spi.IndexSerializer;
+import org.wildfly.clustering.marshalling.jboss.DynamicClassTable;
+import org.wildfly.clustering.marshalling.jboss.ExternalizerObjectTable;
+import org.wildfly.clustering.marshalling.jboss.JBossByteBufferMarshaller;
+import org.wildfly.clustering.marshalling.jboss.SimpleMarshallingConfigurationRepository;
+import org.wildfly.clustering.marshalling.protostream.ProtoStreamByteBufferMarshaller;
+import org.wildfly.clustering.marshalling.protostream.SerializationContextBuilder;
+import org.wildfly.clustering.marshalling.spi.MarshalledValue;
+import org.wildfly.clustering.marshalling.spi.MarshalledValueFactory;
+import org.wildfly.clustering.marshalling.spi.ByteBufferMarshalledValueFactory;
+import org.wildfly.clustering.marshalling.spi.ByteBufferMarshaller;
 import org.wildfly.clustering.server.group.AddressableNode;
 import org.wildfly.clustering.server.logging.ClusteringServerLogger;
 import org.wildfly.clustering.service.concurrent.ServiceExecutor;
@@ -84,25 +92,45 @@ import org.wildfly.security.manager.WildFlySecurityManager;
  * all of which will share the same {@link MessageDispatcher} instance.
  * @author Paul Ferraro
  */
-public class ChannelCommandDispatcherFactory implements AutoCloseableCommandDispatcherFactory, RequestHandler, org.wildfly.clustering.server.group.Group<Address>, MembershipListener, Runnable, Function<GroupListener, ExecutorService> {
+public class ChannelCommandDispatcherFactory implements AutoCloseableCommandDispatcherFactory, RequestHandler, org.wildfly.clustering.spi.group.Group<Address>, MembershipListener, Runnable, Function<GroupListener, ExecutorService> {
+
+    enum MarshallingVersion implements Function<MarshallingConfigurationContext, MarshallingConfiguration> {
+        VERSION_1() {
+            @Override
+            public MarshallingConfiguration apply(MarshallingConfigurationContext context) {
+                MarshallingConfiguration config = new MarshallingConfiguration();
+                ClassLoader userLoader = context.getClassLoader();
+                ClassLoader loader = WildFlySecurityManager.getClassLoaderPrivileged(ChannelCommandDispatcherFactory.class);
+                ClassLoader[] loaders = userLoader.equals(loader) ? new ClassLoader[] { userLoader } : new ClassLoader[] { userLoader, loader };
+                config.setClassResolver(ModularClassResolver.getInstance(context.getModuleLoader()));
+                config.setClassTable(new DynamicClassTable(loaders));
+                config.setObjectTable(new ExternalizerObjectTable(loaders));
+                return config;
+            }
+        },
+        ;
+        static final MarshallingVersion CURRENT = VERSION_1;
+    }
 
     static final Optional<Object> NO_SUCH_SERVICE = Optional.of(NoSuchService.INSTANCE);
     static final ExceptionSupplier<Object, Exception> NO_SUCH_SERVICE_SUPPLIER = Functions.constantExceptionSupplier(NoSuchService.INSTANCE);
 
     private final ConcurrentMap<Address, Node> members = new ConcurrentHashMap<>();
-    private final Map<Object, Map.Entry<Object, Contextualizer>> contexts = new ConcurrentHashMap<>();
+    private final Map<Object, CommandDispatcherContext<?, ?>> contexts = new ConcurrentHashMap<>();
     private final ExecutorService executorService = Executors.newCachedThreadPool(new DefaultThreadFactory(this.getClass()));
     private final ServiceExecutor executor = new StampedLockServiceExecutor();
     private final Map<GroupListener, ExecutorService> listeners = new ConcurrentHashMap<>();
     private final AtomicReference<View> view = new AtomicReference<>();
-    private final MarshallingContext marshallingContext;
+    private final ByteBufferMarshaller marshaller;
     private final MessageDispatcher dispatcher;
     private final Duration timeout;
+    private final ModuleLoader loader;
 
     @SuppressWarnings("resource")
     public ChannelCommandDispatcherFactory(ChannelCommandDispatcherFactoryConfiguration config) {
-        this.marshallingContext = config.getMarshallingContext();
+        this.marshaller = config.getMarshaller();
         this.timeout = config.getTimeout();
+        this.loader = config.getModuleLoader();
         JChannel channel = config.getChannel();
         RequestCorrelator correlator = new RequestCorrelator(channel.getProtocolStack(), this, channel.getAddress()).setMarshaller(new CommandResponseMarshaller(config));
         this.dispatcher = new MessageDispatcher()
@@ -155,7 +183,7 @@ public class ChannelCommandDispatcherFactory implements AutoCloseableCommandDisp
             public void run() {
                 try {
                     response.send(commandTask.get(), false);
-                } catch (Exception e) {
+                } catch (Throwable e) {
                     response.send(e, true);
                 }
             }
@@ -167,34 +195,30 @@ public class ChannelCommandDispatcherFactory implements AutoCloseableCommandDisp
         }
     }
 
-    private ExceptionSupplier<Object, Exception> read(Message message) throws IOException, ClassNotFoundException {
-        try (DataInputStream input = new DataInputStream(new ByteArrayInputStream(message.getRawBuffer(), message.getOffset(), message.getLength()))) {
-            int version = IndexSerializer.VARIABLE.readInt(input);
-            try (Unmarshaller unmarshaller = this.marshallingContext.createUnmarshaller(version)) {
-                unmarshaller.start(Marshalling.createByteInput(input));
-                Object clientId = unmarshaller.readObject();
-                Map.Entry<Object, Contextualizer> entry = this.contexts.get(clientId);
-                if (entry == null) return NO_SUCH_SERVICE_SUPPLIER;
-                Object context = entry.getKey();
-                Contextualizer contextualizer = entry.getValue();
-                @SuppressWarnings("unchecked")
-                Command<Object, Object> command = (Command<Object, Object>) unmarshaller.readObject();
-                // Wrap execution result in an Optional, since command execution might return null
-                ExceptionSupplier<Optional<Object>, Exception> commandExecutionTask = new ExceptionSupplier<Optional<Object>, Exception>() {
-                    @Override
-                    public Optional<Object> get() throws Exception {
-                        return Optional.ofNullable(command.execute(context));
-                    }
-                };
-                ServiceExecutor executor = this.executor;
-                return new ExceptionSupplier<Object, Exception>() {
-                    @Override
-                    public Object get() throws Exception {
-                        return executor.execute(contextualizer.contextualize(commandExecutionTask)).orElse(NO_SUCH_SERVICE).orElse(null);
-                    }
-                };
+    private ExceptionSupplier<Object, Exception> read(Message message) throws IOException {
+        ByteBuffer buffer = ByteBuffer.wrap(message.getRawBuffer(), message.getOffset(), message.getLength());
+        @SuppressWarnings("unchecked")
+        Map.Entry<Object, MarshalledValue<Command<Object, Object>, Object>> entry = (Map.Entry<Object, MarshalledValue<Command<Object, Object>, Object>>) this.marshaller.read(buffer);
+        Object clientId = entry.getKey();
+        CommandDispatcherContext<?, ?> context = this.contexts.get(clientId);
+        if (context == null) return NO_SUCH_SERVICE_SUPPLIER;
+        Object commandContext = context.getCommandContext();
+        Contextualizer contextualizer = context.getContextualizer();
+        MarshalledValue<Command<Object, Object>, Object> value = entry.getValue();
+        Command<Object, Object> command = value.get(context.getMarshalledValueFactory().getMarshallingContext());
+        ExceptionSupplier<Object, Exception> commandExecutionTask = new ExceptionSupplier<Object, Exception>() {
+            @Override
+            public Object get() throws Exception {
+                return context.getMarshalledValueFactory().createMarshalledValue(command.execute(commandContext));
             }
-        }
+        };
+        ServiceExecutor executor = this.executor;
+        return new ExceptionSupplier<Object, Exception>() {
+            @Override
+            public Object get() throws Exception {
+                return executor.execute(contextualizer.contextualize(commandExecutionTask)).orElse(NO_SUCH_SERVICE);
+            }
+        };
     }
 
     @Override
@@ -203,18 +227,57 @@ public class ChannelCommandDispatcherFactory implements AutoCloseableCommandDisp
     }
 
     @Override
-    public <C> CommandDispatcher<C> createCommandDispatcher(Object id, C context) {
-        if (this.contexts.putIfAbsent(id, new AbstractMap.SimpleImmutableEntry<>(context, new DefaultContextualizer())) != null) {
+    public <C> CommandDispatcher<C> createCommandDispatcher(Object id, C commandContext, ClassLoader loader) {
+        ByteBufferMarshaller dispatcherMarshaller = this.createMarshaller(loader);
+        MarshalledValueFactory<ByteBufferMarshaller> factory = new ByteBufferMarshalledValueFactory(dispatcherMarshaller);
+        Contextualizer contextualizer = new DefaultContextualizer();
+        CommandDispatcherContext<C, ByteBufferMarshaller> context = new CommandDispatcherContext<C, ByteBufferMarshaller>() {
+            @Override
+            public C getCommandContext() {
+                return commandContext;
+            }
+
+            @Override
+            public Contextualizer getContextualizer() {
+                return contextualizer;
+            }
+
+            @Override
+            public MarshalledValueFactory<ByteBufferMarshaller> getMarshalledValueFactory() {
+                return factory;
+            }
+        };
+        if (this.contexts.putIfAbsent(id, context) != null) {
             throw ClusteringServerLogger.ROOT_LOGGER.commandDispatcherAlreadyExists(id);
         }
-        CommandMarshaller<C> marshaller = new CommandDispatcherMarshaller<>(this.marshallingContext, id);
-        CommandDispatcher<C> localDispatcher = new LocalCommandDispatcher<>(this.getLocalMember(), context);
-        return new ChannelCommandDispatcher<>(this.dispatcher, marshaller, this, this.timeout, localDispatcher, () -> {
+        CommandMarshaller<C> marshaller = new CommandDispatcherMarshaller<>(this.marshaller, id, factory);
+        CommandDispatcher<C> localDispatcher = new LocalCommandDispatcher<>(this.getLocalMember(), commandContext);
+        return new ChannelCommandDispatcher<>(this.dispatcher, marshaller, dispatcherMarshaller, this, this.timeout, localDispatcher, () -> {
             localDispatcher.close();
             this.contexts.remove(id);
         });
     }
 
+    private ByteBufferMarshaller createMarshaller(ClassLoader loader) {
+        try {
+            return new ProtoStreamByteBufferMarshaller(new SerializationContextBuilder().register(loader).build());
+        } catch (NoSuchElementException e) {
+            ModuleLoader moduleLoader = this.loader;
+            MarshallingConfigurationContext marshallingConfigurationContext = new MarshallingConfigurationContext() {
+                @Override
+                public ClassLoader getClassLoader() {
+                    return loader;
+                }
+
+                @Override
+                public ModuleLoader getModuleLoader() {
+                    return moduleLoader;
+                }
+            };
+            return new JBossByteBufferMarshaller(new SimpleMarshallingConfigurationRepository(MarshallingVersion.class, MarshallingVersion.CURRENT, marshallingConfigurationContext), loader);
+        }
+    }
+
     @Override
     public Registration register(GroupListener listener) {
         this.listeners.computeIfAbsent(listener, this);
