@@ -22,17 +22,22 @@
 
 package org.wildfly.extension.messaging.activemq;
 
-import static org.jboss.as.controller.descriptions.ModelDescriptionConstants.NAME;
-import static org.jboss.as.controller.descriptions.ModelDescriptionConstants.WRITE_ATTRIBUTE_OPERATION;
+import static org.jboss.as.controller.PathElement.pathElement;
 import static org.jboss.as.model.test.ModelTestControllerVersion.EAP_7_0_0;
 import static org.jboss.as.model.test.ModelTestControllerVersion.EAP_7_1_0;
 import static org.junit.Assert.assertTrue;
+import static org.wildfly.extension.messaging.activemq.CommonAttributes.DEFAULT;
+import static org.wildfly.extension.messaging.activemq.CommonAttributes.SERVER;
+import static org.wildfly.extension.messaging.activemq.CommonAttributes.SUBSYSTEM;
 import static org.wildfly.extension.messaging.activemq.MessagingDependencies.getActiveMQDependencies;
 import static org.wildfly.extension.messaging.activemq.MessagingDependencies.getJGroupsDependencies;
 import static org.wildfly.extension.messaging.activemq.MessagingDependencies.getMessagingActiveMQGAV;
+import static org.wildfly.extension.messaging.activemq.MessagingExtension.ADDRESS_SETTING_PATH;
 import static org.wildfly.extension.messaging.activemq.MessagingExtension.BRIDGE_PATH;
 import static org.wildfly.extension.messaging.activemq.MessagingExtension.CLUSTER_CONNECTION_PATH;
 import static org.wildfly.extension.messaging.activemq.MessagingExtension.CONNECTION_FACTORY_PATH;
+import static org.wildfly.extension.messaging.activemq.MessagingExtension.EXTERNAL_JMS_QUEUE_PATH;
+import static org.wildfly.extension.messaging.activemq.MessagingExtension.EXTERNAL_JMS_TOPIC_PATH;
 import static org.wildfly.extension.messaging.activemq.MessagingExtension.POOLED_CONNECTION_FACTORY_PATH;
 import static org.wildfly.extension.messaging.activemq.MessagingExtension.REPLICATION_COLOCATED_PATH;
 import static org.wildfly.extension.messaging.activemq.MessagingExtension.REPLICATION_MASTER_PATH;
@@ -43,10 +48,8 @@ import java.io.IOException;
 import java.util.List;
 import java.util.Properties;
 
-import org.jboss.as.clustering.controller.Operations;
 import org.jboss.as.controller.ModelVersion;
 import org.jboss.as.controller.PathAddress;
-import org.jboss.as.controller.PathElement;
 import org.jboss.as.controller.security.CredentialReference;
 import org.jboss.as.model.test.FailedOperationTransformationConfig;
 import org.jboss.as.model.test.ModelTestControllerVersion;
@@ -56,6 +59,7 @@ import org.jboss.as.subsystem.test.AdditionalInitialization;
 import org.jboss.as.subsystem.test.KernelServices;
 import org.jboss.as.subsystem.test.KernelServicesBuilder;
 import org.jboss.dmr.ModelNode;
+import org.junit.Assert;
 import org.junit.Test;
 import org.wildfly.clustering.jgroups.spi.JGroupsDefaultRequirement;
 import org.wildfly.clustering.spi.ClusteringDefaultRequirement;
@@ -66,20 +70,28 @@ import org.wildfly.extension.messaging.activemq.jms.ConnectionFactoryAttributes;
 /**
  *  * @author <a href="http://jmesnil.net/">Jeff Mesnil</a> (c) 2012 Red Hat inc
  */
-public class MessagingActiveMQSubsystem_3_0_TestCase extends AbstractSubsystemBaseTest {
+public class MessagingActiveMQSubsystem_6_0_TestCase extends AbstractSubsystemBaseTest {
 
-    public MessagingActiveMQSubsystem_3_0_TestCase() {
+    public MessagingActiveMQSubsystem_6_0_TestCase() {
         super(MessagingExtension.SUBSYSTEM_NAME, new MessagingExtension());
     }
 
     @Override
     protected String getSubsystemXml() throws IOException {
-        return readResource("subsystem_3_0.xml");
+        return readResource("subsystem_6_0.xml");
     }
 
     @Override
     protected String getSubsystemXsdPath() throws IOException {
-        return "schema/wildfly-messaging-activemq_3_0.xsd";
+        return "schema/wildfly-messaging-activemq_6_0.xsd";
+    }
+
+    @Override
+    protected String[] getSubsystemTemplatePaths() throws IOException {
+        return new String[] {
+                "/subsystem-templates/messaging-activemq.xml",
+                "/subsystem-templates/messaging-activemq-colocated.xml",
+        };
     }
 
     @Override
@@ -94,18 +106,36 @@ public class MessagingActiveMQSubsystem_3_0_TestCase extends AbstractSubsystemBa
     protected KernelServices standardSubsystemTest(String configId, boolean compareXml) throws Exception {
         return super.standardSubsystemTest(configId, false);
     }
+
+    @Test
+    public void testJournalAttributes() throws Exception {
+        KernelServices kernelServices = standardSubsystemTest(null, false);
+        ModelNode rootModel = kernelServices.readWholeModel();
+        ModelNode serverModel = rootModel.require(SUBSYSTEM).require(MessagingExtension.SUBSYSTEM_NAME).require(SERVER)
+                .require(DEFAULT);
+
+        Assert.assertEquals(1357, serverModel.get(ServerDefinition.JOURNAL_BUFFER_TIMEOUT.getName()).resolve().asInt());
+        Assert.assertEquals(102400, serverModel.get(ServerDefinition.JOURNAL_FILE_SIZE.getName()).resolve().asInt());
+        Assert.assertEquals(2, serverModel.get(ServerDefinition.JOURNAL_MIN_FILES.getName()).resolve().asInt());
+        Assert.assertEquals(5, serverModel.get(ServerDefinition.JOURNAL_POOL_FILES.getName()).resolve().asInt());
+    }
+
     /////////////////////////////////////////
     //  Tests for HA Policy Configuration  //
     /////////////////////////////////////////
 
     @Test
     public void testHAPolicyConfiguration() throws Exception {
-        standardSubsystemTest("subsystem_3_0_ha-policy.xml");
+        standardSubsystemTest("subsystem_6_0_ha-policy.xml");
     }
 
     ///////////////////////
     // Transformers test //
     ///////////////////////
+    @Test
+    public void testTransformersEAP_7_2_0() throws Exception {
+        testTransformers(ModelTestControllerVersion.EAP_7_2_0, MessagingExtension.VERSION_4_0_0);
+    }
 
     @Test
     public void testTransformersEAP_7_1_0() throws Exception {
@@ -117,6 +147,11 @@ public class MessagingActiveMQSubsystem_3_0_TestCase extends AbstractSubsystemBa
         testTransformers(EAP_7_0_0, MessagingExtension.VERSION_1_0_0);
     }
 
+    @Test
+    public void testRejectingTransformersEAP_7_2_0() throws Exception {
+        testRejectingTransformers(ModelTestControllerVersion.EAP_7_2_0, MessagingExtension.VERSION_4_0_0);
+    }
+
     @Test
     public void testRejectingTransformersEAP_7_1_0() throws Exception {
         testRejectingTransformers(EAP_7_1_0, MessagingExtension.VERSION_2_0_0);
@@ -130,7 +165,7 @@ public class MessagingActiveMQSubsystem_3_0_TestCase extends AbstractSubsystemBa
     private void testTransformers(ModelTestControllerVersion controllerVersion, ModelVersion messagingVersion) throws Exception {
         //Boot up empty controllers with the resources needed for the ops coming from the xml to work
         KernelServicesBuilder builder = createKernelServicesBuilder(createAdditionalInitialization())
-                .setSubsystemXmlResource("subsystem_3_0_transform.xml");
+                .setSubsystemXmlResource("subsystem_6_0_transform.xml");
         builder.createLegacyKernelServicesBuilder(createAdditionalInitialization(), controllerVersion, messagingVersion)
                 .addMavenResourceURL(getMessagingActiveMQGAV(controllerVersion))
                 .addMavenResourceURL(getActiveMQDependencies(controllerVersion))
@@ -143,6 +178,7 @@ public class MessagingActiveMQSubsystem_3_0_TestCase extends AbstractSubsystemBa
         assertTrue(mainServices.getLegacyServices(messagingVersion).isSuccessfulBoot());
 
         checkSubsystemModelTransformation(mainServices, messagingVersion);
+        mainServices.shutdown();
     }
 
     private void testRejectingTransformers(ModelTestControllerVersion controllerVersion, ModelVersion messagingVersion) throws Exception {
@@ -159,7 +195,7 @@ public class MessagingActiveMQSubsystem_3_0_TestCase extends AbstractSubsystemBa
         assertTrue(mainServices.isSuccessfulBoot());
         assertTrue(mainServices.getLegacyServices(messagingVersion).isSuccessfulBoot());
 
-        List<ModelNode> ops = builder.parseXmlResource("subsystem_3_0_reject_transform.xml");
+        List<ModelNode> ops = builder.parseXmlResource("subsystem_6_0_reject_transform.xml");
         System.out.println("ops = " + ops);
         PathAddress subsystemAddress = PathAddress.pathAddress(SUBSYSTEM_PATH);
 
@@ -179,12 +215,26 @@ public class MessagingActiveMQSubsystem_3_0_TestCase extends AbstractSubsystemBa
                                 ServerDefinition.JOURNAL_LARGE_MESSAGES_TABLE,
                                 ServerDefinition.JOURNAL_PAGE_STORE_TABLE,
                                 ServerDefinition.JOURNAL_DATABASE,
-                                ServerDefinition.JOURNAL_JDBC_NETWORK_TIMEOUT))
+                                ServerDefinition.JOURNAL_JDBC_NETWORK_TIMEOUT,
+                                ServerDefinition.JOURNAL_JDBC_LOCK_EXPIRATION,
+                                ServerDefinition.JOURNAL_JDBC_LOCK_RENEW_PERIOD,
+                                ServerDefinition.JOURNAL_NODE_MANAGER_STORE_TABLE,
+                                ServerDefinition.JOURNAL_FILE_OPEN_TIMEOUT,
+                                ServerDefinition.GLOBAL_MAX_DISK_USAGE,
+                                ServerDefinition.DISK_SCAN_PERIOD,
+                                ServerDefinition.GLOBAL_MAX_MEMORY_SIZE
+                        ))
                 .addFailedAttribute(subsystemAddress.append(SERVER_PATH, REPLICATION_MASTER_PATH),
                         new ChangeToTrueConfig(HAAttributes.CHECK_FOR_LIVE_SERVER.getName()))
                 .addFailedAttribute(subsystemAddress.append(SERVER_PATH, REPLICATION_COLOCATED_PATH, MessagingExtension.CONFIGURATION_MASTER_PATH),
                         new ChangeToTrueConfig(HAAttributes.CHECK_FOR_LIVE_SERVER.getName()))
-                .addFailedAttribute(subsystemAddress.append(SERVER_PATH, PathElement.pathElement(CommonAttributes.HTTP_CONNECTOR)),
+                .addFailedAttribute(subsystemAddress.append(SERVER_PATH, ADDRESS_SETTING_PATH),
+                        new FailedOperationTransformationConfig.NewAttributesConfig(
+                                AddressSettingDefinition.AUTO_CREATE_QUEUES,
+                                AddressSettingDefinition.AUTO_DELETE_QUEUES,
+                                AddressSettingDefinition.AUTO_CREATE_ADDRESSES,
+                                AddressSettingDefinition.AUTO_DELETE_ADDRESSES))
+                .addFailedAttribute(subsystemAddress.append(SERVER_PATH, pathElement(CommonAttributes.HTTP_CONNECTOR)),
                         new FailedOperationTransformationConfig.NewAttributesConfig(
                                 HTTPConnectorDefinition.SERVER_NAME))
                 .addFailedAttribute(subsystemAddress.append(SERVER_PATH, BRIDGE_PATH),
@@ -196,7 +246,8 @@ public class MessagingActiveMQSubsystem_3_0_TestCase extends AbstractSubsystemBa
                 .addFailedAttribute(subsystemAddress.append(SERVER_PATH, CONNECTION_FACTORY_PATH),
                         new FailedOperationTransformationConfig.NewAttributesConfig(
                                 ConnectionFactoryAttributes.Common.DESERIALIZATION_BLACKLIST,
-                                ConnectionFactoryAttributes.Common.DESERIALIZATION_WHITELIST))
+                                ConnectionFactoryAttributes.Common.DESERIALIZATION_WHITELIST,
+                                ConnectionFactoryAttributes.Common.INITIAL_MESSAGE_PACKET_SIZE))
                 .addFailedAttribute(subsystemAddress.append(SERVER_PATH, POOLED_CONNECTION_FACTORY_PATH),
                         new FailedOperationTransformationConfig.NewAttributesConfig(
                                 ConnectionFactoryAttributes.Pooled.ALLOW_LOCAL_TRANSACTIONS,
@@ -206,13 +257,54 @@ public class MessagingActiveMQSubsystem_3_0_TestCase extends AbstractSubsystemBa
                                 ConnectionFactoryAttributes.Common.DESERIALIZATION_BLACKLIST,
                                 ConnectionFactoryAttributes.Common.DESERIALIZATION_WHITELIST))
                 ;
+        } else if (messagingVersion.equals(MessagingExtension.VERSION_2_0_0)) {
+            config.addFailedAttribute(subsystemAddress.append(SERVER_PATH, ADDRESS_SETTING_PATH),
+                        new FailedOperationTransformationConfig.NewAttributesConfig(
+                            AddressSettingDefinition.AUTO_CREATE_QUEUES,
+                            AddressSettingDefinition.AUTO_DELETE_QUEUES,
+                            AddressSettingDefinition.AUTO_CREATE_ADDRESSES,
+                            AddressSettingDefinition.AUTO_DELETE_ADDRESSES))
+                .addFailedAttribute(subsystemAddress.append(SERVER_PATH),
+                        new FailedOperationTransformationConfig.NewAttributesConfig(ServerDefinition.JOURNAL_JDBC_LOCK_EXPIRATION,
+                                ServerDefinition.JOURNAL_JDBC_LOCK_RENEW_PERIOD,
+                                ServerDefinition.JOURNAL_NODE_MANAGER_STORE_TABLE,
+                                ServerDefinition.JOURNAL_FILE_OPEN_TIMEOUT,
+                                ServerDefinition.GLOBAL_MAX_DISK_USAGE,
+                                ServerDefinition.DISK_SCAN_PERIOD,
+                                ServerDefinition.GLOBAL_MAX_MEMORY_SIZE
+                        ))
+                .addFailedAttribute(subsystemAddress.append(SERVER_PATH, CONNECTION_FACTORY_PATH),
+                        new FailedOperationTransformationConfig.NewAttributesConfig(
+                                ConnectionFactoryAttributes.Common.INITIAL_MESSAGE_PACKET_SIZE));
+        } else if (messagingVersion.compareTo(MessagingExtension.VERSION_5_0_0) > 0 ) {
+            config.addFailedAttribute(subsystemAddress.append(SERVER_PATH),
+                    new FailedOperationTransformationConfig.NewAttributesConfig(
+                            ServerDefinition.GLOBAL_MAX_DISK_USAGE,
+                            ServerDefinition.DISK_SCAN_PERIOD,
+                            ServerDefinition.GLOBAL_MAX_MEMORY_SIZE,
+                            ServerDefinition.JOURNAL_FILE_OPEN_TIMEOUT));
+        } else if (messagingVersion.compareTo(MessagingExtension.VERSION_6_0_0) > 0 ) {
+            config.addFailedAttribute(subsystemAddress.append(SERVER_PATH), new FailedOperationTransformationConfig.NewAttributesConfig(ServerDefinition.JOURNAL_FILE_OPEN_TIMEOUT));
         }
 
-        config.addFailedAttribute(subsystemAddress.append(SERVER_PATH, MessagingExtension.BROADCAST_GROUP_PATH), new FailedOperationTransformationConfig.NewAttributesConfig(BroadcastGroupDefinition.JGROUPS_CHANNEL));
-        config.addFailedAttribute(subsystemAddress.append(SERVER_PATH, DiscoveryGroupDefinition.PATH), new FailedOperationTransformationConfig.NewAttributesConfig(DiscoveryGroupDefinition.JGROUPS_CHANNEL));
-        config.addFailedAttribute(subsystemAddress.append(DiscoveryGroupDefinition.PATH), FailedOperationTransformationConfig.REJECTED_RESOURCE);
-
+        if (messagingVersion.compareTo(MessagingExtension.VERSION_4_0_0) > 0) {
+            config.addFailedAttribute(subsystemAddress.append(SERVER_PATH, MessagingExtension.BROADCAST_GROUP_PATH), new FailedOperationTransformationConfig.NewAttributesConfig(BroadcastGroupDefinition.JGROUPS_CHANNEL));
+            config.addFailedAttribute(subsystemAddress.append(SERVER_PATH, DiscoveryGroupDefinition.PATH), new FailedOperationTransformationConfig.NewAttributesConfig(DiscoveryGroupDefinition.JGROUPS_CHANNEL));
+            config.addFailedAttribute(subsystemAddress.append(SERVER_PATH, MessagingExtension.QUEUE_PATH), new FailedOperationTransformationConfig.NewAttributesConfig(QueueDefinition.ROUTING_TYPE));
+            config.addFailedAttribute(subsystemAddress.append(DiscoveryGroupDefinition.PATH), FailedOperationTransformationConfig.REJECTED_RESOURCE);
+            config.addFailedAttribute(subsystemAddress.append(pathElement(CommonAttributes.REMOTE_CONNECTOR)), FailedOperationTransformationConfig.REJECTED_RESOURCE);
+            config.addFailedAttribute(subsystemAddress.append(pathElement(CommonAttributes.IN_VM_CONNECTOR)), FailedOperationTransformationConfig.REJECTED_RESOURCE);
+            config.addFailedAttribute(subsystemAddress.append(pathElement(CommonAttributes.CONNECTOR)), FailedOperationTransformationConfig.REJECTED_RESOURCE);
+            config.addFailedAttribute(subsystemAddress.append(MessagingExtension.HTTP_CONNECTOR_PATH), FailedOperationTransformationConfig.REJECTED_RESOURCE);
+            config.addFailedAttribute(subsystemAddress.append(CONNECTION_FACTORY_PATH), FailedOperationTransformationConfig.REJECTED_RESOURCE);
+            config.addFailedAttribute(subsystemAddress.append(POOLED_CONNECTION_FACTORY_PATH), FailedOperationTransformationConfig.REJECTED_RESOURCE);
+            config.addFailedAttribute(subsystemAddress.append(EXTERNAL_JMS_QUEUE_PATH), FailedOperationTransformationConfig.REJECTED_RESOURCE);
+            config.addFailedAttribute(subsystemAddress.append(EXTERNAL_JMS_TOPIC_PATH), FailedOperationTransformationConfig.REJECTED_RESOURCE);
+        } else if (messagingVersion.compareTo(MessagingExtension.VERSION_6_0_0) > 0) {
+            config.addFailedAttribute(subsystemAddress.append(SERVER_PATH, MessagingExtension.QUEUE_PATH), new FailedOperationTransformationConfig.NewAttributesConfig(QueueDefinition.ROUTING_TYPE));
+        }
         ModelTestUtils.checkFailedTransformedBootOperations(mainServices, messagingVersion, ops, config);
+        mainServices.shutdown();
     }
 
     @Override
@@ -225,38 +317,4 @@ public class MessagingActiveMQSubsystem_3_0_TestCase extends AbstractSubsystemBa
                 CredentialReference.CREDENTIAL_STORE_CAPABILITY + ".cs1",
                 Capabilities.DATA_SOURCE_CAPABILITY + ".fooDS");
     }
-
-    private static class ChangeToTrueConfig extends FailedOperationTransformationConfig.AttributesPathAddressConfig<ChangeToTrueConfig> {
-
-        private final String attribute;
-
-        ChangeToTrueConfig(String attribute) {
-            super(attribute);
-            this.attribute = attribute;
-        }
-
-        @Override
-        protected boolean isAttributeWritable(String attributeName) {
-            return true;
-        }
-
-        @Override
-        protected boolean checkValue(ModelNode operation, String attrName, ModelNode attribute, boolean isGeneratedWriteAttribute) {
-            if (!isGeneratedWriteAttribute && Operations.getName(operation).equals(WRITE_ATTRIBUTE_OPERATION) && operation.hasDefined(NAME) && operation.get(NAME).asString().equals(this.attribute)) {
-                // The attribute won't be defined in the :write-attribute(name=<attribute name>,.. boot operation so don't reject in that case
-                return false;
-            }
-            return !attribute.equals(ModelNode.TRUE);
-        }
-
-        @Override
-        protected boolean checkValue(String attrName, ModelNode attribute, boolean isWriteAttribute) {
-            throw new IllegalStateException();
-        }
-
-        @Override
-        protected ModelNode correctValue(ModelNode toResolve, boolean isWriteAttribute) {
-            return ModelNode.TRUE;
-        }
-    }
 }
