@@ -211,6 +211,36 @@ public class ClientCompatibilityUnitTestCase {
         testWF("1.0.1.Final", 9990);
     }
 
+    @Test
+    public void testCore210Final() throws Exception {
+        testWF("2.1.0.Final", 9999);
+    }
+
+    @Test
+    public void testCore210FinalHttp() throws Exception {
+        testWF("2.1.0.Final", 9990);
+    }
+
+    @Test
+    public void testCore221Final() throws Exception {
+        testWF("2.2.1.Final", 9999);
+    }
+
+    @Test
+    public void testCore221FinalHttp() throws Exception {
+        testWF("2.2.1.Final", 9990);
+    }
+
+    @Test
+    public void testCore3010Final() throws Exception {
+        testWF("3.0.10.Final", 9999);
+    }
+
+    @Test
+    public void testCore3010FinalHttp() throws Exception {
+        testWF("3.0.10.Final", 9990);
+    }
+
     @Test
     public void testCurrent() throws Exception {
         test(ModelControllerClient.Factory.create(CONTROLLER_ADDRESS, 9999));
@@ -275,7 +305,6 @@ public class ClientCompatibilityUnitTestCase {
 
         final ChildFirstClassLoaderBuilder classLoaderBuilder = new ChildFirstClassLoaderBuilder(false);
         classLoaderBuilder.addRecursiveMavenResourceURL(artifact + ":" + version, excludes);
-        classLoaderBuilder.addParentFirstClassPattern("org.jboss.as.controller.client.ModelControllerClientConfiguration");
         classLoaderBuilder.addParentFirstClassPattern("org.jboss.as.controller.client.ModelControllerClient");
         classLoaderBuilder.addParentFirstClassPattern("org.jboss.as.controller.client.OperationMessageHandler");
         classLoaderBuilder.addParentFirstClassPattern("org.jboss.as.controller.client.Operation");
